"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = exports.applyApplication = void 0;
const koa_1 = __importDefault(require("koa"));
const router_1 = require("./router");
const config_1 = require("./config");
function applyApplication(application, req, res) {
    application.applyRequest(req, res);
}
exports.applyApplication = applyApplication;
class Application extends koa_1.default {
    static isApplication(obj) {
        return obj instanceof Application;
    }
    static isApplicationClass(objClass) {
        return objClass.__sugar_Application;
    }
    constructor() {
        super();
        this.Controllers = [];
        const thisConstructor = this.constructor;
        thisConstructor.Controllers.forEach((Controller) => {
            this.useController(Controller);
        });
        thisConstructor.Applications.forEach((Application) => {
            this.useApplication(Application);
        });
        this.config = new config_1.Config();
        if (thisConstructor.defaultConfig) {
            this.config.add(thisConstructor.defaultConfig);
        }
    }
    onError(err, ctx) {
        if (!ctx.res.writableEnded &&
            !ctx.res.writableFinished) {
            let statusCode = 500;
            if (typeof err.statusCode === 'number') {
                statusCode = err.statusCode;
            }
            this.emit('appError', err, ctx);
            ctx.status = statusCode;
            ctx.body = {
                code: err.code || 0,
                message: err.message
            };
        }
    }
    useApplication(ApplicationClass) {
        const { application, routerMiddleware } = (0, router_1.appendApplication)(ApplicationClass);
        this.use(routerMiddleware);
    }
    useController(ControllerClass) {
        const { router } = (0, router_1.appendControllerToRouter)(ControllerClass);
        if (router) {
            this.use(router.routes());
        }
        this.Controllers.push(ControllerClass);
    }
    createContext(req, res) {
        const ctx = super.createContext.call(this, req, res);
        ctx.routerPath = req.routerPath;
        return ctx;
    }
    createApply() {
        this._applyRequest = this.callback();
    }
    applyRequest(req, res) {
        return this._applyRequest(req, res);
    }
}
exports.Application = Application;
Application.__sugar_Application = true;
Application.Controllers = [];
Application.Applications = [];
//# sourceMappingURL=application.js.map