"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cookie = exports.bodyText = exports.bodyFormData = exports.bodyJSON = exports.body = exports.params = exports.header = exports.query = exports.Context = exports.NodeRequest = exports.Request = exports.NodeResponse = exports.Response = exports.config = exports.createParameterGetter = exports.getter = void 0;
require("reflect-metadata");
const co_body_1 = __importDefault(require("co-body"));
const parameterGetterMetadataKey = Symbol('_parameterGetter');
function getter(target, propertyName, descriptor) {
    let method = descriptor.value;
    descriptor.value = function (...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const ctx = this.context;
            let existingParameterGetters = Reflect.getOwnMetadata(parameterGetterMetadataKey, target, propertyName);
            let gotValues = [];
            if (existingParameterGetters) {
                for (let parameterGetter of existingParameterGetters) {
                    gotValues[parameterGetter.index] = yield parameterGetter.getter(ctx);
                }
            }
            return method && method.call(this, ...gotValues, ctx, ...args);
        });
    };
}
exports.getter = getter;
function createParameterGetter(getterCallback) {
    return function parameterValidate(target, propertyKey, parameterIndex) {
        let existingParameterGetters = Reflect.getOwnMetadata(parameterGetterMetadataKey, target, propertyKey) || [];
        existingParameterGetters.push({
            index: parameterIndex,
            getter: getterCallback
        });
        Reflect.defineMetadata(parameterGetterMetadataKey, existingParameterGetters, target, propertyKey);
    };
}
exports.createParameterGetter = createParameterGetter;
function config(configKey, propertyKey, parameterIndex) {
    if (typeof configKey === 'string') {
        return createParameterGetter((ctx) => {
            return ctx.app.config.get(configKey);
        });
    }
    else if (typeof configKey === 'object' && propertyKey) {
        return createParameterGetter((ctx) => {
            return ctx.app.config;
        })(configKey, propertyKey, parameterIndex);
    }
}
exports.config = config;
exports.Response = createParameterGetter((ctx) => {
    return ctx.response;
});
exports.NodeResponse = createParameterGetter((ctx) => {
    return ctx.res;
});
exports.Request = createParameterGetter((ctx) => {
    return ctx.request;
});
exports.NodeRequest = createParameterGetter((ctx) => {
    return ctx.req;
});
exports.Context = createParameterGetter((ctx) => {
    return ctx;
});
const query = (key) => createParameterGetter((ctx) => {
    return ctx.query[key];
});
exports.query = query;
const header = (key) => createParameterGetter((ctx) => {
    return ctx.headers[key];
});
exports.header = header;
const params = (key) => createParameterGetter((ctx) => {
    return ctx.params[key];
});
exports.params = params;
const body = (path) => createParameterGetter((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const parsedBodyJSON = yield getParsedBody(ctx, () => ((0, co_body_1.default)(ctx.req)));
    return findDataByPath(parsedBodyJSON, path);
}));
exports.body = body;
const bodyJSON = (path) => createParameterGetter((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const parsedBodyJSON = yield getParsedBody(ctx, () => (co_body_1.default.json(ctx.req)));
    return findDataByPath(parsedBodyJSON, path);
}));
exports.bodyJSON = bodyJSON;
const bodyFormData = (path) => createParameterGetter((ctx) => __awaiter(void 0, void 0, void 0, function* () {
    const parsedBodyJSON = yield getParsedBody(ctx, () => (co_body_1.default.form(ctx.req)));
    return findDataByPath(parsedBodyJSON, path);
}));
exports.bodyFormData = bodyFormData;
const bodyText = () => createParameterGetter((ctx) => {
    return co_body_1.default.text(ctx.req);
});
exports.bodyText = bodyText;
function findDataByPath(data, path) {
    let result = data;
    if (typeof path === 'string') {
        const pathArr = path.split('.');
        for (let i = 0; i < pathArr.length; i++) {
            result = result[pathArr[i]];
            if (typeof result === 'undefined'
                || result === null) {
                break;
            }
        }
    }
    return result;
}
function getParsedBody(ctx, parseFn) {
    return __awaiter(this, void 0, void 0, function* () {
        let parsedBodyJSON;
        if (ctx._parsedBodyJSON) {
            parsedBodyJSON = ctx._parsedBodyJSON;
        }
        else {
            parsedBodyJSON = yield parseFn();
            ctx._parsedBodyJSON = parsedBodyJSON;
        }
        return parsedBodyJSON;
    });
}
const cookie = (...args) => createParameterGetter((ctx) => {
    return ctx.cookies.get(...args);
});
exports.cookie = cookie;
//# sourceMappingURL=parameter.js.map