"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.appendControllerToRouter = exports.appendApplication = exports.getRoutesFromController = exports.AllRoute = exports.DelRoute = exports.PutRoute = exports.PostRoute = exports.GetRoute = exports.create = void 0;
const url_1 = __importDefault(require("url"));
const koa_router_1 = __importDefault(require("koa-router"));
const controller_1 = require("./controller");
const logger = __importStar(require("../shared/logger"));
const create_thunk_descriptor_1 = require("../shared/create-thunk-descriptor");
/**
 * create routes
 */
exports.create = (0, create_thunk_descriptor_1.createThunkAttributeDecorator)((options, target, key, descriptor) => {
    if (controller_1.Controller.isController(target)) {
        if (!Object.getOwnPropertyDescriptor(target, controller_1.ROUTES_KEY)) {
            Object.defineProperty(target, controller_1.ROUTES_KEY, {
                configurable: true,
                enumerable: false,
                writable: true,
                value: []
            });
        }
        target[controller_1.ROUTES_KEY].push({
            key: key,
            method: options.method,
            path: options.path
        });
    }
});
function GetRoute(path) {
    return (0, exports.create)({ method: 'get', path });
}
exports.GetRoute = GetRoute;
function PostRoute(path) {
    return (0, exports.create)({ method: 'post', path });
}
exports.PostRoute = PostRoute;
function PutRoute(path) {
    return (0, exports.create)({ method: 'put', path });
}
exports.PutRoute = PutRoute;
function DelRoute(path) {
    return (0, exports.create)({ method: 'del', path });
}
exports.DelRoute = DelRoute;
function AllRoute(path) {
    return (0, exports.create)({ method: 'all', path });
}
exports.AllRoute = AllRoute;
function getRoutesFromController(controller) {
    return controller[controller_1.ROUTES_KEY];
}
exports.getRoutesFromController = getRoutesFromController;
function appendApplication(ApplicationClass) {
    const application = new ApplicationClass();
    application.createApply();
    const { path, host, rewrite } = ApplicationClass;
    const routerMiddleware = (ctx, next) => __awaiter(this, void 0, void 0, function* () {
        const { req, res } = ctx;
        let reqUrl = url_1.default.parse(req.url || '');
        let reqHost = req.headers.host;
        let current = true;
        if (!reqUrl.pathname) {
            current = false;
        }
        else if (!path) {
            current = false;
        }
        else if (typeof path === 'string'
            ? (reqUrl.pathname.indexOf(path) !== 0)
            : !path.test(reqUrl.pathname)) {
            current = false;
        }
        if (current && host) {
            if (!reqHost) {
                current = false;
            }
            else {
                if (typeof host === 'string') {
                    if (reqHost !== host) {
                        current = false;
                    }
                }
                else {
                    if (host.includes) {
                        if (host.includes.indexOf(reqHost) === -1) {
                            current = false;
                        }
                    }
                    if (host.excludes) {
                        if (host.excludes.indexOf(reqHost) !== -1) {
                            current = false;
                        }
                    }
                }
            }
        }
        if (current) {
            let routerPath;
            if (rewrite) {
                routerPath = rewrite(reqUrl, req);
            }
            else if (path) {
                if (typeof path === 'string') {
                    routerPath = reqUrl.pathname.slice(path.length);
                }
                else {
                    routerPath = reqUrl.pathname.replace(path, '');
                }
                if (routerPath.length === 0) {
                    routerPath = '/';
                }
            }
            if (routerPath) {
                req.routerPath = routerPath;
            }
            else {
                req.routerPath = reqUrl.pathname;
            }
            yield application.applyRequest(req, res);
        }
        yield next();
    });
    return {
        application,
        routerMiddleware
    };
}
exports.appendApplication = appendApplication;
function appendControllerToRouter(ControllerClass) {
    let router = new koa_router_1.default();
    let prefix = ControllerClass.prefix;
    const routes = ControllerClass.prototype[controller_1.ROUTES_KEY];
    if (routes) {
        if (prefix) {
            router.prefix(prefix);
        }
        routes.forEach(({ key, method, path }) => {
            const methodRegister = router[method];
            if (methodRegister) {
                methodRegister.call(router, path, (ctx, next) => __awaiter(this, void 0, void 0, function* () {
                    const controller = new ControllerClass();
                    controller.context = ctx;
                    if (typeof controller[key] === 'function') {
                        const controllerReturn = yield controller[key].call(controller, next);
                        if (typeof controllerReturn !== 'undefined' &&
                            !ctx.res.writableEnded &&
                            !ctx.res.writableFinished) {
                            ctx.body = controllerReturn;
                        }
                    }
                    yield next();
                }));
            }
        });
        logger.success(`register routes \n Controller: ${ControllerClass.name} \n prefix: ${prefix} \n routes: ${JSON.stringify(routes)}`);
    }
    return {
        router
    };
}
exports.appendControllerToRouter = appendControllerToRouter;
//# sourceMappingURL=router.js.map