"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createParameterValidate = exports.validated = exports.validate = exports.array = exports.number = exports.string = exports.required = void 0;
require("reflect-metadata");
const create_thunk_descriptor_1 = require("../shared/create-thunk-descriptor");
const error_1 = require("./error");
const parameterValidateMetadataKey = Symbol('parameterValidate');
exports.required = createParameterValidate((value, parameterIndex) => {
    if (!value) {
        throw new error_1.SugarServerError(400, `param [${parameterIndex}] is required`, {
            statusCode: 400
        });
    }
});
exports.string = createParameterValidate((value, parameterIndex) => {
    if (typeof value !== 'string') {
        throw new error_1.SugarServerError(400, `param [${parameterIndex}] ${value} not string`, {
            statusCode: 400
        });
    }
});
exports.number = createParameterValidate((value, parameterIndex) => {
    if (typeof value !== 'number') {
        throw new error_1.SugarServerError(400, `param [${parameterIndex}] ${value} not number`, {
            statusCode: 400
        });
    }
});
const array = (itemExtraCheck) => createParameterValidate((value, parameterIndex, ctx) => __awaiter(void 0, void 0, void 0, function* () {
    if (!Array.isArray(value)) {
        throw new error_1.SugarServerError(400, `param [${parameterIndex}] ${value} not array`, {
            statusCode: 400
        });
    }
    if (itemExtraCheck) {
        let itemValidateCallback;
        if (isParameterValidate(itemExtraCheck)) {
            itemValidateCallback = itemExtraCheck.validateCallback;
        }
        else {
            itemValidateCallback = itemExtraCheck;
        }
        for (let i = 0; i < value.length; i++) {
            let itemValue = value[i];
            yield itemValidateCallback(itemValue, `${parameterIndex}-${i}`, ctx);
        }
    }
}));
exports.array = array;
function isParameterValidate(obj) {
    return !!obj.validateCallback;
}
function validate(target, propertyName, descriptor) {
    let method = descriptor.value;
    descriptor.value = function () {
        return __awaiter(this, arguments, void 0, function* () {
            // 通过parameterGetter处理后，最后2个参数是 ctx、next
            const ctx = arguments[arguments.length - 2];
            let existingParameterValidator = Reflect.getOwnMetadata(parameterValidateMetadataKey, target, propertyName);
            if (existingParameterValidator) {
                for (let parameterValidator of existingParameterValidator) {
                    yield parameterValidator.validator(arguments[parameterValidator.index], parameterValidator.index, ctx);
                }
            }
            return method && method.apply(this, arguments);
        });
    };
}
exports.validate = validate;
exports.validated = (0, create_thunk_descriptor_1.createThunkAttributeDecorator)((validators, target, key, descriptor) => {
    let value = descriptor.value;
    if (typeof value === 'function') {
        for (let i = 0; i < validators.length; i++) {
            let parameterValidator = validators[i];
            if (Array.isArray(parameterValidator)) {
                parameterValidator.forEach((item) => {
                    item(target, key, i);
                });
            }
            else {
                parameterValidator(target, key, i);
            }
        }
        return validate(target, key, descriptor);
    }
});
function createParameterValidate(validateCallback) {
    function parameterValidate(target, propertyKey, parameterIndex) {
        let existingParameterValidator = Reflect.getOwnMetadata(parameterValidateMetadataKey, target, propertyKey) || [];
        existingParameterValidator.push({
            index: parameterIndex,
            validator: validateCallback
        });
        Reflect.defineMetadata(parameterValidateMetadataKey, existingParameterValidator, target, propertyKey);
    }
    parameterValidate.validateCallback = validateCallback;
    return parameterValidate;
}
exports.createParameterValidate = createParameterValidate;
//# sourceMappingURL=validator.js.map